#!/usr/bin/env python3

"""
Command Line Utility to connect or redirect ports to a Cloud Virtual Machine
"""


import os
import sys

from dynacli import main as dynamain

cwd = os.path.dirname(os.path.realpath(__file__))


search_path = [cwd]
sys.path.extend(search_path)

_map = {
    "__version__": "1.0.1b1",
    "__doc__": """
CLI tool for Computational Biology""",
}


def _set_main_attrs(**kwargs):
    _main = sys.modules["__main__"]
    for key, val in kwargs.items():
        setattr(_main, key, val)


# For package distro purposes
def main():
    _set_main_attrs(**_map)
    dynamain(search_path)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3


"""
Common errors defined here:
"""


class WrongFormatError(Exception):
    """Raised when the input file is not in the correct format"""

    def __init__(self, *args, **kwargs):
        default_message = (
            "The file you passed is not supported! Only .txt files are accepted."
        )

        # If any arguments are passed...
        if args:
            # ... pass them to the super constructor
            super().__init__(*args, **kwargs)
        else:
            # ... pass the default message to the super constructor
            super().__init__(default_message, **kwargs)


class EmptyFileError(Exception):
    """Raised when the input file is empty"""

    def __init__(self, *args, **kwargs):
        default_message = "The file you passed is EMPTY!"

        # If any arguments are passed...
        if args:
            # ... pass them to the super constructor
            super().__init__(*args, **kwargs)
        else:
            # ... pass the default message to the super constructor
            super().__init__(default_message, **kwargs)


class WrongNucleotideError(Exception):
    """Raised when the input file contains wrong nucleotides"""

    def __init__(self, *args, **kwargs):
        default_message = "The file you passed contains wrong set of nucleotides!"

        # If any arguments are passed...
        if args:
            # ... pass them to the super constructor
            super().__init__(*args, **kwargs)
        else:
            # ... pass the default message to the super constructor
            super().__init__(default_message, **kwargs)



#!/usr/bin/env python3


"""
Count feature to count the nucleotides and patterns in the given DNA or RNA sequence.
"""


from collections import Counter
from typing import Dict

from biocli._common._sequence import Sequence

__all__ = ["nucleotide", "pattern", "frequency"]


def nucleotide(text_file: str) -> Counter:
    """
    Counts all the nucleotides in the sequence separately.

    Args:
        text_file (str): The full file path to the sequence file.

    Return:  Nucleotides and their counts.
    """

    seq = Sequence(text_file)

    counts = Counter([seq])

    for symbol, count in sorted(counts.items()):
        print(f"{symbol} -- {count}")

    return counts


def pattern(pattern_: str, text_file: str) -> str:
    """
    Counts the number of occurance of the pattern in given text file.

    Args:
        pattern (str): The pattern to search for.
        text_file (str): The full file path to the sequence file.

    Return: Count of pattern.
    """

    seq = Sequence(text_file)

    return f"{str(seq).count(pattern_)} patterns found."


def frequency(window_len: int, text_file: str) -> Dict:
    """

    Counts all the possible nucleotide combinations of given length in the sequence.

    Args:
        text_file (str): The full file path to the sequence file
        window_len (int): The length of the nucleotide combination.

    Return: Nucleotide combinations and their counts.
    """

    seq = Sequence(text_file)

    frequency_ = {}

    for i in range(len(seq) - window_len + 1):
        pattern_ = seq[i : i + window_len]
        frequency_[pattern_] = 0

        for j in range(len(seq) - window_len + 1):
            if seq[j : (j + window_len)] == pattern_:
                frequency_[pattern_] += 1

    for freq, value in frequency_.items():
        print(f"{freq} -- {value}")

    return frequency_



#!/usr/bin/env python3

"""
Helper functions defined here.
"""
from datetime import datetime
from typing import Union


def _new_name(seq_type: str) -> Union[str, None]:

    """
    Creates a new file name based on the sequence type.
    """

    tstamp = datetime.now().strftime("%Y_%m_%d-%I-%M-%S_%p")

    if seq_type.lower() == "dna":
        return f"dna_sequence_{tstamp}.txt"
    if seq_type.lower() == "rna":
        return f"rna_sequence_{tstamp}.txt"
    print("Invalid sequence type. Choose RNA or DNA.")
    return "Failed!"


def save(seq_type: str, text: str) -> None:
    """
    Save the sequence to a file.
    """

    with open(str(_new_name(seq_type)), "w+", encoding="utf-8") as file:
        file.write(text)
    print("Saved successfully!")



#!/usr/bin/env python3


"""
Translate feature to translate the given DNA sequence to RNA and vice versa.
"""


from biocli._common._errors import WrongNucleotideError
from biocli._common._helper import save
from biocli._common._sequence import Sequence

__all__ = ["dna_to_rna", "rna_to_dna"]


def dna_to_rna(text_file: str, to_file: bool = False) -> str:
    """
    Translates the DNA sequence to RNA.

    Args:
        text_file (str): The full file path to the sequence file.
        to_file (bool): Save to file option. Default is False.

    Return: RNA sequence
    """

    seq = Sequence(text_file)

    # check if the sequence is DNA
    if seq.is_rna():
        raise WrongNucleotideError

    rna_text = "".join(["U" if x == "T" else x for x in seq])

    if to_file:
        save("RNA", rna_text)
    else:
        print(rna_text)

    return rna_text


def rna_to_dna(text_file: str, to_file: bool = False) -> str:
    """
    Translates the DNA sequence to RNA.

    Args:
        text_file (str): The full file path to the sequence file.
        to_file (bool): Save to file option. Default is False.

    Return: DNA sequence
    """

    seq = Sequence(text_file)

    # check if the sequence is RNA
    if seq.is_dna():
        raise WrongNucleotideError

    dna_text = "".join(["T" if x == "U" else x for x in seq])

    if to_file:
        save("DNA", dna_text)
    else:
        print(dna_text)

    return dna_text



#!/usr/bin/env python3


"""
Sequence class defined here.
"""
from typing import Any, Union

from ._errors import EmptyFileError, WrongFormatError

# if text file not provided we use the default value
DEFAULT_SAMPLE = "../samples/sample.txt"


# loads the text file and returns the sequence as a string
def _load_sequence(text_file: str = DEFAULT_SAMPLE) -> Union[str, Any]:
    try:
        with open(text_file, "r", encoding="utf-8") as file:
            symbol_list = [file.read()]
            return "".join(symbol_list).replace("\n", "")
    except WrongFormatError as err:
        if not text_file.endswith(".txt"):
            raise err
        return "Failed!"


class Sequence:
    """
    Class for DNA and RNA sequences and their methods.
    """

    def __init__(self, text_file: str):
        self.__file = text_file
        self.seq = _load_sequence(self.__file)
        if self.is_empty():
            raise EmptyFileError

    def __repr__(self):
        return self.seq

    def __eq__(self, other):
        return self.seq == other.seq

    def __hash__(self):
        return hash(self.seq)

    def __len__(self):
        return len(self.seq)

    def __iter__(self):
        return iter(self.seq)

    def __getitem__(self, slice_: Union[int, slice]):
        return self.seq[slice_]

    def __str__(self) -> str:
        return self.seq

    def is_dna(self) -> bool:
        """checks if the sequence is DNA"""
        return "U" not in self.seq

    def is_rna(self) -> bool:
        """checks if the sequence is RNA"""
        return "T" not in self.seq

    def is_empty(self) -> bool:
        """checks if the sequence is empty"""
        return len(self.seq) == 0



#!/usr/bin/env python3


"""
Pattern feature for finding given pattern in the given DNA or RNA sequence.
"""


from biocli._common._sequence import Sequence

__all__ = ["match"]


def match(pattern: str, text_file: str) -> list:
    """
    Checks for the presence of a pattern in the sequence.

    Args:
        pattern (str): The pattern to search for.
        text_file (str): The full file path to the sequence file.

    Return: Position indices of the pattern in the sequence.
    """

    seq = Sequence(text_file)

    positions = []

    for i in range(len(seq) - len(pattern) + 1):
        if seq[i : i + (len(pattern))] == pattern.upper():
            positions.append(i)

    print(positions)
    return positions
